#           DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#                   Version 2, December 2004
#
# Copyright (C) 2020 Ryosuke Abe <chike@sfc.wide.ad.jp>
# 
# Everyone is permitted to copy and distribute verbatim or modified
# copies of this license document, and changing it is allowed as long
# as the name is changed.
#
#           DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
#  TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
#
# 0. You just DO WHAT THE FUCK YOU WANT TO.
import os

from hashstorage.utils import *

class HashStorage():
    def __init__(self, datadir=None):
        self.datadir = ".hashstorage" if datadir is None else  datadir
        self.storage = {}
        self._loadDataDir()

    def _loadDataDir(self):
        if not os.path.exists(self.datadir):
            os.makedirs(self.datadir)
        #TODO: loadDataDir
    
    def store(self, value):
        key = sha256(value)
        self.storage[key] = value
        saveJson(self.storage, self.datadir+"/storage.json")
        return key
    
    def get(self, key):
        if key not in self.storage:
            return None
        else:
            return self.storage[key]

    def shutdown(self):
        saveJson(self.storage, self.datadir+"/storage.json")
        



from hashstorage.hashstorage import *
import sys
import argparse

from flask import Flask, request, jsonify, make_response
app = Flask(__name__)
hs = None

class HashStorageCli():
    def __init__(self, args):
        self.parser = self.parseArgs(args)

    def parseArgs(self, args):
        parser = argparse.ArgumentParser(description='Simple Hash Storage v0.0.1 (C) Ryosuke Abe 2020')
        parser.add_argument('-dir', '--datadir', help='Path to datadir') 
        return parser.parse_args(args)

@app.route("/", methods=["GET"])
def versionMessage():
    return 'Simple Hash Storage v0.0.1 (C) Ryosuke Abe 2020'

@app.route("/store", methods=['POST'])
def GetStoreReq():
    params = request.json
    if "value" not in params.keys():
        resbody = {"result": False, "error": "Request Json is Invalid"} 
    else:
        data = params["value"]
        key = hs.store(data)

        resbody = {"result": True, "key": key}
    return make_response(jsonify(resbody))
    
@app.route("/get", methods=['POST'])
def GetGetReq():
    params = request.json
    data = hs.get(params["key"])
    if data:
        resbody = {"result": True, "key": data}
    else:
        resbody = {"result": False, "error": "Key (%s) is not found"%params["key"] }
    return make_response(jsonify(resbody))

def main():
    runHashStorageCli(sys.argv[1:])

def runHashStorageCli(args):
    cli = HashStorageCli(args)
    global hs 
    hs = HashStorage(cli.parser.datadir)
    try:
        app.run(host="localhost", port=6666)
    finally:
        hs.shutdown()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hashstorage']

package_data = \
{'': ['*']}

install_requires = \
['Flask==1.1.1']

entry_points = \
{'console_scripts': ['hashstorage-cli = hashstorage.cli:main']}

setup_kwargs = {
    'name': 'hashstorage',
    'version': '0.1.0',
    'description': 'Implementation of Simple-HashStorage.',
    'long_description': None,
    'author': 'chike0905',
    'author_email': 'chike@sfc.wide.ad.jp',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/chike0905/simple-hashstorage',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import hashlib
import binascii
import json

def sha256(keyword): 
    bytekey = binascii.hexlify(keyword.encode("utf-8"))
    hashvalue = hashlib.sha256(bytekey).digest()
    return hashvalue.hex()

def loadJson(jsonpath):
    try:
        with open(jsonpath) as f:
            df = json.load(f)
    except FileNotFoundError:
        return False
    return df

def saveJson(dictdata, storepath):
    f = open(storepath, "w")
    json.dump(dictdata, f,
            ensure_ascii=False,
            indent=4,
            sort_keys=True,
            separators=(',', ': '))
